// const can be used in the global scope.
const THREE_HOURS_IN_SECS: u32 = 60 * 60 * 3;

fn main() {
    // From ch3.1

    // this code wont compile.
    // let x = 5;
    // println!("The value of x is: {x}");
    // // x = 6; // This line is illegal. Can't reassign a non-mutable variable.
    // println!("The value of x is: {x}");

    //------------------------------------------------------------------------

    // Let can only be used inside of a function.
    // This makes sense due to the memory freeing paradigm.
    let mut y = 5;
    println!("The value of y is: {y}"); // Don't need python f string for this!
    y = 6;
    println!("The value of y is: {y}");

    //------------------------------------------------------------------------

    let x = 5;

    let x = x + 1;

    {
        let x = x * 2;
        println!("The value of x in inner scope is: {x}");
    }

    println!("The value of x is: {x}");

    //------------------------------------------------------------------------

    // This compiles
    let spaces = "   ";
    let spaces = spaces.len();
    println!("spaces: {spaces}");

    // This doesn't
    // let mut spacesi = "   ";
    // spacesi = spacesi.len();

    //------------------------------------------------------------------------
    // From ch3.2

    // Different ways to declare an integer
    let decimal_num: isize = 98_222;
    let hex_num: isize = 0xff;
    let octal_num: isize = 0o77;
    let binary_num: isize = 0b1111_0000;
    let byte_num: u8 = b'A';

    println!("decimal num: {decimal_num}");
    println!("hex num: {hex_num}");
    println!("octal num: {octal_num}");
    println!("binary num: {binary_num}");
    println!("byte num: {byte_num}");

    //------------------------------------------------------------------------

    // floating point types
    let a = 2.0;
    let b: f32 = 3.0;

    //------------------------------------------------------------------------

    // boolean types
    let t = true;
    let f: bool = false;

    //------------------------------------------------------------------------

    // char types
    let c = 'z';
    let z: char = 'ℤ';
    let heart_eyed_cat = '😻';

    //------------------------------------------------------------------------

    // tuples :)
    let tup: (i32, f64, u8) = (500, 6.4, 1);
    let (i, j, k) = tup;
    println!("The value of j is: {j}");

    let five_hundred = tup.0;
    let six_point_four = tup.1;
    let one = tup.2;

    let unit = (); // This is essentially rust's null. Similar to Haskell's version.

    let mut mut_tup: (i32, i32) = (1, 2);
    mut_tup.0 = 0;
    mut_tup.1 += 5;
    println!("mut_tup: ({}, {})", mut_tup.0, mut_tup.1);

    //------------------------------------------------------------------------

    // Arrays, rust arrays have fixed length.
    // Best when want data allocated on stack rather than heap.
    // Vector type is more flexible.
    let arr = [1, 2, 3, 4, 5];

    // array of same values repeated an amount of times. Functional roots pog
    let arr = [3; 5];

    // [type; number of elements]
    let arr: [i32; 5] = [1, 2, 3, 4, 5];

    let first = arr[0];
    let second = arr[1];

    // Rust will throw an indexing runtime error after it compilies.
    // let tenth = arr[10];

    //------------------------------------------------------------------------
    // ch3.3
    another_function(3);
    print_labeled_measurement(10, 'c');

    //------------------------------------------------------------------------

    // This errors. Statements can't return values.
    // let x = (let y = 6);

    //------------------------------------------------------------------------
    // The {...} below is an expression. The last line in the expression has
    // no semicolon. If there is a semicolon, it turns the line into a statement
    // and thus won't return a value.
    let t = {
        let x = 3;
        x + 1
    };
    println!("Value of t is: {t}");

    //------------------------------------------------------------------------
    let num = plus_one(five());
    println!("The value of num is: {num}");

    //------------------------------------------------------------------------
    // ch3.4 About commments

    /*
        This is a multiline comment pog
        Documentation comments are covered in chapter 14.
    */

    //------------------------------------------------------------------------
    // ch3.5 control flow

    // let number = 3;
    let number = 7;

    // The condition must evaluate to a bool.
    if number < 5 {
        println!("condition true");
    } else {
        println!("condition false");
    }

    if number != 0 {
        println!("Number is not 0");
    }

    let new_num = 6;

    if new_num % 4 == 0 {
        println!("new_num is divisible by 4");
    } else if new_num % 3 == 0 {
        println!("new_num divisible by 3");
    } else if new_num % 2 == 0 {
        println!("new_num divisible by 2");
    } else {
        println!("new_num not divisible by 4, 3, or 2");
    }

    //------------------------------------------------------------------------

    let condition = true;
    let cond_num = if condition { 5 } else { 6 }; // can't mismatch types this way.
    println!("The value of cond_num is: {cond_num}");

    //------------------------------------------------------------------------
    let val = if condition { 1 } else { 2 };
    let new_val;
    if condition {
        // referencing new_val from above within this if statement scope.
        // Still a bit iffy on how the memory management works.
        new_val = 1;
    } else {
        new_val = 2
    };
    println!("new_val = {new_val}");

    //------------------------------------------------------------------------
    // Looping syntax

    let mut counter = 0;
    // can grab a value from a loop statement
    let result = loop {
        println!("again!");
        if counter == 10 {
            break counter * 2;
        }
        counter += 1;
    };
    println!("The result is: {result}");

    //------------------------------------------------------------------------
    // loop labels :eyes:

    let mut count = 0;
    'counting_up: loop {
        // clojure where :eyes:
        println!("count = {count}");
        let mut remaining = 10;

        loop {
            println!("remaining = {remaining}");
            if remaining == 9 {
                break;
            }
            if count == 2 {
                break 'counting_up;
            }
            remaining -= 1;
        }
        count += 1;
    }
    println!("End count = {count}");

    //------------------------------------------------------------------------
    // While loop

    let mut while_num = 3;

    while while_num != 0 {
        println!("while_num: {while_num}");
        while_num -= 1;
    }
    println!("LIFTOFFFFFFF (while_num reached 0)");

    //------------------------------------------------------------------------
    // For loop

    let for_arr = [10, 20, 30, 40, 50];
    let mut index = 0;

    // The old, unpog way of doing this
    while index < 5 {
        println!("while value is: {}", for_arr[index]);
        index += 1;
    }

    // The new, pog way of doing this
    for element in for_arr {
        println!("for value is: {element}");
    }

    // for loop recreation of liftoff above
    for pog_num in (1..4).rev() {
        println!("{pog_num}");
    }
    println!("for LIFTOFFFFFF!");

    //------------------------------------------------------------------------
    // Summary questions at the end of the ch3.5

    let ftc = fahrenheit_to_celsius(32.1);
    println!("fahrenheit to celsius of 32.0 degress F: {ftc}");
    let ctf = celsius_to_fahrenheit(5.0);
    println!("celsius_to_fahrenheitof 5.0 degrees C: {ctf}");
    let fib_num = fibonacci(5);
    println!("fibonacci of fib_num: {fib_num}");
}

// Statements: Instructions that perform some action and don't return anything
//   like the statements inside of these functions. The function definitions themselves
//   are also statements.
// Expressions: Evaluate to a resultant value.
// Sounds like Haskell.

// all functions with parameters must have a type attached to it.
fn another_function(x: i32) {
    // ch3.3
    println!("Another function.");
    println!("The value of x is: {x}");
}

fn print_labeled_measurement(value: i32, unit_label: char) {
    println!("The measurement is: {value}{unit_label}");
}

// literally returns 5 with the arrow. Very functional
// Rust. I love this language.
fn five() -> i32 {
    5
}

fn plus_one(x: i32) -> i32 {
    x + 1
}

// ch3.5 exercise
fn fahrenheit_to_celsius(temp: f64) -> f64 {
    (temp - 32.0) * (5.0 / 9.0)
}

// ch3.5 exercise
fn celsius_to_fahrenheit(temp: f64) -> f64 {
    (9.0 / 5.0) * temp + 32.0
}

fn fibonacci(num: usize) -> usize {
    if num == 0 {
        return 0;
    } else if num == 1 {
        return 1;
    }
    fibonacci(num - 1) + fibonacci(num - 2)
}

// I'm not programming the christmas carol song
